/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.konfiguration;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein Strassenteilsegment modelliert.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class StrassenTeilSegment extends DavProxyObjekt {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug _debug = Debug.getLogger();

	/**
	 * Länge des Straßenteilsegments.
	 */
	private long _laenge = 0;

	/**
	 * Anzahl Fahrstreifen.
	 */
	private int _anzahlFahrStreifen = 0;

	/**
	 * Steigung/Gefälle.
	 */
	private String _steigungGefaelle = null;

	/**
	 * Offset innerhalb des Straßensegments.
	 */
	private long _offset = 0;

	/**
	 * Erstellt eine neue Instanz vom Typ <code>StrassenSegment</code>.
	 * 
	 * @param davObjekt
	 *            das zuzuordnente Dav-Objekt
	 * @param laenge
	 *            die Länge in m des Strassensegments
	 * @param anzahlFahrStreifen
	 *            Anzahl Fahrstreifen
	 * @param steigungGefaelle
	 *            Steigung/Gefälle
	 * @param offset
	 *            Offset innerhalb des Straßensegments
	 */
	public StrassenTeilSegment(SystemObject davObjekt, long laenge, int anzahlFahrStreifen, String steigungGefaelle,
			long offset) {
		super(davObjekt);
		_laenge = laenge;
		_anzahlFahrStreifen = anzahlFahrStreifen;
		_steigungGefaelle = steigungGefaelle;
		_offset = offset;
	}

	/**
	 * @return die Länge in m dieses Strassenteilsegments
	 */
	public long getLaenge() {
		return _laenge;
	}

	/**
	 * @return die Anzahl der Fahrstreifen dieses Strassenteilsegments
	 */
	public int getAnzahlFahrStreifen() {
		return _anzahlFahrStreifen;
	}

	/**
	 * @return Steigung oder Gefälle dieses Strassenteilsegments
	 */
	public String getSteigungGefaelle() {
		return _steigungGefaelle;
	}

	/**
	 * @return den Offset dieses Strassenteilsegments auf seinem Strassensegment
	 */
	public long getOffset() {
		return _offset;
	}

}
