/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.test;

import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class Prueffall2 extends PrueffallBasis implements Runnable {
	/**
	 * Debug-Logger für Logging-Ausgaben
	 */
	private final static Debug _debug = Debug.getLogger();

	private final static String BAUSTELLE_MENGE = "Baustellen";
	private final static String BAUSTELLE_PID_PREFIX = "situation.Baustelle";
	private final static String BAUSTELLE_NAME_PREFIX = "Baustelle";

	private boolean _appStarten = false;

	public Prueffall2(ClientDavInterface connection, boolean appStarten) {
		super(connection);

		_appStarten = appStarten;

		Runtime.getRuntime().addShutdownHook(new ShutdownHook());
	}

	public Prueffall2(ClientDavInterface connection) {
		this(connection, false);
	}

	public class ShutdownHook extends Thread {
		public void run() {
			if (getApplikationsProzess() != null) {
				getApplikationsProzess().destroy();
				System.out.println("Zu prüfende Applikation gestoppt");
			}
			parameterRestaurieren();
			bereinigeBaustellen();
		}
	}

	public void run() {
		_debug.info("Beginn der Prüfungen Prüffall 2\nStarte P.2.1");
		setzteIndikatoren(new int[] { 3, 3, 3, 4, 3, 4, 4, 4, 4, 5, 4, 5, 5, 1, 4, 3, 4, 1 });
		bereinigeBaustellen();
		erzeugeBaustellen();

		if (_appStarten) {
			try {
				starteApplikationsProzess();
				System.out.println("Zu prüfende Applikation gestartet");

				Thread.sleep(PrueffallUmgebung.APP_START_VERZOEGERUNG);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		} else {
			System.out.println("RDS/TMC-Meldungserstellung jetzt extern starten");
			warteAufEingabe();
		}

		_debug.info("Testzeitpunkt: t = 720s");

		warte(30000);
		_debug.info("Starte P.2.2\nTestzeitpunkt: t = 750s");
		aendereParameter1();

		warte(30000);
		_debug.info("Testzeitpunkt: t = 780s");
		aendereParameter2();

		warte(40000);
		_debug.info("Testzeitpunkt: t = 820s");
		setzteIndikatoren(new int[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0 });
		_debug.info("Ende der Prüfungen für Prüffall 2");

	}

	private void aendereParameter1() {
		AttributeGroup atg = getConnection().getDataModel().getAttributeGroup(MODELL_ATG);
		Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");

		Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp))
				.getData();
		if (altParam != null) {
			this.setOriginalParameter(altParam);

			Data neuParam = altParam.createModifiableCopy();
			neuParam.getItem("RDSBerücksichtigeBaustellen").asTextValue().setText("Ja");

			asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
			new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam)
					.start();
			_debug.config("Parameter-Aktualisierung gestartet");
			_debug.info("Parameteränderungen:" + "\n\tRDSBerücksichtigeBaustellen: Ja");

		}
	}

	private void aendereParameter2() {
		AttributeGroup atg = getConnection().getDataModel().getAttributeGroup(MODELL_ATG);
		Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");

		Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp))
				.getData();
		if (altParam != null) {
			Data neuParam = altParam.createModifiableCopy();
			neuParam.getItem("RDSBaustelleSuchbereichVor").asUnscaledValue().set(1000);

			asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
			new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam)
					.start();
			_debug.config("Parameter-Aktualisierung gestartet");
			_debug.info("Parameteränderungen:" + "\n\tRDSBaustelleSuchbereichVor: 1000m");
		}
	}

	private void erzeugeBaustellen() {
		DataModel dm = getConnection().getDataModel();
		MutableSet menge = ((ConfigurationObject) getUmgebung().getVmn()).getMutableSet(BAUSTELLE_MENGE);
		short ext = 1;

		for (int i = 0; i < 2; i += 1) {
			SystemObject baustelle = null;

			while (ext < Short.MAX_VALUE && dm.getObject(BAUSTELLE_PID_PREFIX + ext) != null) {
				ext += 1;
			}

			DynamicObjectType typ = (DynamicObjectType) dm.getType("typ.baustelle");
			try {
				baustelle = getDefaultConfigurationArea().createDynamicObject(typ, BAUSTELLE_PID_PREFIX + ext,
						BAUSTELLE_NAME_PREFIX + ext, null);
				menge.add(baustelle);
			} catch (ConfigurationChangeException e) {
				_debug.warning("Fehler", e);
			}

			if (baustelle != null) {
				String[] segmente = new String[0];
				long startOffset = -1;
				long endOffset = -1;

				switch (i) {
				case 0:
					segmente = new String[] { "ass.p.11906.p.11907.0" };
					startOffset = 450;
					endOffset = 1520;
					break;
				case 1:
					segmente = new String[] { "ass.p.11906.p.11907.0", "ass.p.11907.p.11908.0" };
					startOffset = 1850;
					endOffset = 600;
					break;
				}
				if (segmente.length > 0) {
					AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften");
					Data param = getConnection().createData(atg);
					ReferenceArray array = param.getItem("StraßenSegment").asReferenceArray();
					array.setLength(segmente.length);
					for (int j = 0; j < segmente.length; j += 1) {
						array.getReferenceValue(j).setSystemObject(dm.getObject(segmente[j]));
					}
					param.getItem("StartOffset").asUnscaledValue().set(startOffset);
					param.getItem("EndOffset").asUnscaledValue().set(endOffset);
					param.getItem("StartZeit").asTimeValue().setMillis(getConnection().getTime());
					param.getItem("Dauer").asTimeValue().setSeconds(0);

					new ParameterSender(getConnection(), baustelle,
							new DataDescription(atg, dm.getAspect("asp.parameterVorgabe")), param).start();
				}

				AttributeGroup atg = dm.getAttributeGroup("atg.baustellenEigenschaften");
				Data param = getConnection().createData(atg);
				param.getItem("Status").asTextValue().setText("gültig");
				param.getItem("RestKapazität").asTextValue().setText("nicht ermittelbar");
				param.getItem("Veranlasser").asTextValue().setText("VRZ-intern");

				new ParameterSender(getConnection(), baustelle,
						new DataDescription(atg, dm.getAspect("asp.parameterVorgabe")), param).start();

				_debug.info(
						"Neues dynamisches Baustellenobjekt " + baustelle.getName() + " erzeugt"
								+ (segmente.length > 0 ? ":\n\tStart auf " + segmente[0] + " bei " + startOffset
										+ "m\n\tEnde auf " + segmente[segmente.length - 1] + " bei " + endOffset + "m"
										: ""));
			}
		}
	}

	private void bereinigeBaustellen() {
		MutableSet menge = ((ConfigurationObject) getUmgebung().getVmn()).getMutableSet(BAUSTELLE_MENGE);
		int n = 0;

		List<SystemObject> liste = menge.getElements();

		if (liste.isEmpty()) {
			return;
		}

		for (SystemObject baustelle : liste) {
			if (baustelle.getPid().startsWith(BAUSTELLE_PID_PREFIX)) {
				try {
					menge.remove(baustelle);
					((DynamicObject) baustelle).invalidate();
					n += 1;
				} catch (ConfigurationChangeException e) {
					_debug.error("Fehler", e);
				}

			}
		}
		_debug.config(n + " Baustellen-Objekt(e) bereinigt");
	}

}
