/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.test;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

public class RDSTMCTest implements StandardApplication {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private Debug _debug;

	private int _pruefFall;

	/**
	 * Default- Konstruktor.<br>
	 * Wird typischerweise von <code>main</code> aufgerufen
	 */
	public RDSTMCTest() {
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Überschriebene Methode von <code>StandardApplication</code>, die die
	 * speziellen Startparameter auswertet.
	 * 
	 * @param argumentList
	 *            siehe
	 *            sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
	 */
	public void parseArguments(ArgumentList argumentList) {
		_debug = Debug.getLogger();

		_pruefFall = argumentList.fetchArgument("-Prüffall=1").intValueBetween(1, 2);

		argumentList.fetchUnusedArguments();
	}

	/**
	 * Überschriebene Methode von StandardApplication, die die Initialisierung
	 * durchführt.<br>
	 * Es wird eine Instanz von <code>VerwaltungEngstellen</code> erzeugt und
	 * initialisiert
	 * 
	 * @param connection
	 *            siehe
	 *            sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	public void initialize(ClientDavInterface connection) {
		try {
			switch (_pruefFall) {
			case 1:
				new Thread(new Prueffall1(connection)).start();
				break;
			case 2:
				new Thread(new Prueffall2(connection)).start();
				break;
			}

			_debug.info("Tests für RDS/TMC initialisiert");
		} catch (Exception e) {
			_debug.error("Abbruch der Tests für RDS/TMC während der Initialisierung", e);
			System.exit(1);
		}
	}

	/**
	 * Programmeinstieg.
	 * 
	 * @param arguments
	 *            Argumente
	 */
	public static void main(String[] arguments) {
		StandardApplicationRunner.run(new RDSTMCTest(), arguments);
	}

}
