/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.verkehrsanalyse;

import de.bsvrz.ste.tmc.konfiguration.StrassenAbschnitt;
import de.bsvrz.ste.tmc.konfiguration.StrassenTeilSegment;

/**
 * Modelliert einen lokalen Störfall in Bezug auf einen Messquerschnitt.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class StreckenbezogenerStoerfall extends Stoerfall {

	/**
	 * Instanziiert einen streckenbezogenen Störfall für einen Strassenabschnitt
	 * auf einem Strassensegment.
	 * 
	 * @param sa
	 *            der Strassenabschnitt, für den der lokale Störfall
	 *            instanziiert werden soll
	 * @param stoerfallSituation
	 *            {@link StoerfallSituation}
	 */
	public StreckenbezogenerStoerfall(StrassenAbschnitt sa, StoerfallSituation stoerfallSituation) {
		super(sa, stoerfallSituation);

		_startOffset = getSegment().getMessQuerschnitte().contains(sa.getVonMq()) ? sa.getVonMq().getOffset() : 0; // getSegment().getLaenge();
		_endOffset = getSegment().getMessQuerschnitte().contains(sa.getNachMq()) ? sa.getNachMq().getOffset()
				: getSegment().getLaenge();
	}

	public StreckenbezogenerStoerfall(StrassenTeilSegment sts, StoerfallSituation stoerfallSituation) {
		super(sts, stoerfallSituation);

		_startOffset = sts.getOffset();
		_endOffset = _startOffset + sts.getLaenge();
	}

	/**
	 * Startoffset in m auf dem Straßensegment dieses Stöfalls.
	 */
	private long _startOffset;

	/**
	 * Endoffset in m auf dem Straßensegment dieses Stöfalls.
	 */
	private long _endOffset;

	/**
	 * @return den Startoffset in m auf dem Straßensegment dieses Störfalls
	 */
	long getStartOffset() {
		long result = _startOffset;
		result = result < 0 ? 0 : result;
		result = result > getSegment().getLaenge() ? getSegment().getLaenge() : result;
		return result;
	}

	/**
	 * @return den Endoffset in m auf dem Straßensegment dieses Störfalls
	 */
	long getEndOffset() {
		long result = _endOffset;
		result = result < 0 ? 0 : result;
		result = result > getSegment().getLaenge() ? getSegment().getLaenge() : result;
		return result;
	}

}
